#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#define SIZEX 80
#define SIZEY 15

Display *dpy;            
Window root, win;
int scrn;
char color[10]="gray";
char color_bg[10]="darkblue";
char spool_file[25];

GC gc,gc2,gc3,gc4;
Font font;
XImage *bg;
XFontStruct* font_;
char *window_name={"BadMail"}; 
char  mail_info[25] = {0,};
int x,y,width,height;
char draw_border, center, transparent, fix;
static time_t last_check;
static time_t last_stat;
static time_t time_till_check;
int new_mail, total_mail, reading;
int blinking;

char* get_data(char ret[]){
	// Stat the spool file first
	struct stat buf;
	FILE *fp;

	// Is it time to check?
	if(time(NULL) >= (last_check + time_till_check)){
		last_check = time(NULL);

		// Get mail
		new_mail = total_mail = 0;
		fp = fopen(spool_file, "r");
		if(fp == NULL) {
			fprintf(stderr, "BadMail: Can't open %s\n", spool_file);
			exit(0);
		}
		
		while(!feof(fp)){
			char buf[128];
			if(fgets(buf, 128, fp) == NULL) break;
			if(!strncmp(buf, "From ", 5)) {
				total_mail++;
				if(reading) new_mail++;
				else reading = 1;
			}else{
				if(reading && !strncmp(buf, "Status:", 7)) {
					if(strchr(buf+7, 'R') == NULL) new_mail++;
					reading = 0;
					continue;
				}
			}
		}	
		sprintf(ret,"Mail: %d:%d", new_mail, total_mail);
	}
}

void drawBadMail(){
	int xoff;

	get_data(mail_info);

	if(transparent)
		XPutImage(dpy, win, gc, bg, 0,0,0,0,width,height);
	else
		XClearWindow(dpy, win);

	// Oke, We got our data, and if New_mail is > 0 we will blink.
	if(new_mail > 0){
		if(blinking){
			XFillRectangle(dpy, win, gc, 1, 1, width-2, height-2);
			blinking=0;
		}else{
			blinking=1;
		}	
	}

	// Calculate offset total - strlen
	if(center){
		xoff = width - XTextWidth(font_, mail_info, strlen(mail_info));	
		xoff -= xoff/2;
	}else{
		// Not centering
		xoff = 2;
	}

	// Plot the string
	if(blinking && new_mail>0) XDrawString(dpy, win, gc, xoff, 12, mail_info, strlen(mail_info));
	else if(new_mail>0) XDrawString(dpy, win, gc2, xoff, 12, mail_info, strlen(mail_info));
	else XDrawString(dpy, win, gc, xoff, 12, mail_info, strlen(mail_info));

    // Draw borders
    if(draw_border) {
	    XDrawLine(dpy, win, gc3, 0, 0, width, 0);
        XDrawLine(dpy, win, gc3, 0, 0, 0, height);      
        XDrawLine(dpy, win, gc4, 1, height-1, width, height-1);
        XDrawLine(dpy, win, gc4, width-1, 1, width-1, height);  
	}
}

unsigned long GetColor(char *color)
{  
  Colormap cmap;
  XColor c0, c1;
  cmap = DefaultColormap(dpy, 0);
  XAllocNamedColor(dpy, cmap, color, &c1, &c0);
  return(c1.pixel);
} 

void GetBackground(){
    // Get the image under our window
	bg = XGetImage(dpy, root, x, y, width, height, -1, ZPixmap);
}

unsigned long MakeBorderColor(char *color, char fgbg){
        Colormap cmap;
        char bordercolor[8] = {0,};
        XColor c0, c1;
        unsigned short r,g,b;

        cmap = DefaultColormap(dpy, 0);
        XAllocNamedColor(dpy, cmap, color, &c1, &c0);
        r = c1.red;
        g = c1.green;
        b = c1.blue;
        
        // If leftupper
        if(fgbg == 1){
                if(r+20000 <= 65535 && g+20000 <= 56635 && b+20000 <= 56635){
                        r+=20000; g+=20000; b+=20000;
                }else{
                        // Just white
                        r = g = b = 56635;
                }
        }
        // If bottomright
        else{
                if(r-40000 >= 0 && g-40000 >= 0 && b-40000 >= 0){
                        r-=40000; g-=40000; b-=40000;
                }else{
                        // Just black
                        r = g = b = 0;
                }
        }

        r = r/256;
        g = g/256;
        b = b/256;
        
        sprintf(bordercolor, "#%lX%lX%lX",r,g,b);
        
        XAllocNamedColor(dpy, cmap, bordercolor, &c1, &c0);
        return c1.pixel;
}

void win_create(){
    XSetWindowAttributes attr;
    XSizeHints szhints;
    unsigned long mask;
    
    if(fix){ 
        attr.border_pixel = BlackPixel(dpy,scrn);
        attr.background_pixel = GetColor(color_bg);
        attr.override_redirect = True;                                                                 
        attr.event_mask = NULL;
        mask = CWBackPixel|CWBorderPixel|CWOverrideRedirect;
        win = XCreateWindow(dpy,root,x,y,width,height,0,DefaultDepth(dpy,scrn),InputOutput,CopyFromParent,mask,&attr);

        szhints.flags = PPosition | PSize;
        szhints.x = x;
        szhints.y = y;
        szhints.width = width;
        szhints.height = height;
        
        XSetNormalHints(dpy,win,&szhints);
    }else{
        win = XCreateSimpleWindow(dpy, root, x, y, width, height, 0, 0, GetColor(color_bg));
    }
}

int main(int argc, char **argv){	
	int i;
	char myFont[70];
	char *mail_file;
	width = SIZEX;
	height = SIZEY;
	x = 1;
	y = 1;
	time_till_check = 30; // 30 seconds is def
	last_check = time(NULL);
	draw_border = center = 1;
	transparent = fix = 0;

	sprintf(myFont, "-misc-fixed-*-*-*-*-18-*-*-*-*-*-*-*");
	mail_file = getenv("MAIL");
	sprintf(spool_file, "%s", mail_file);
	
	/* Parse argv */
    for (i=1; i < argc; i++) {
       if (!strcmp(argv[i],"-g") || !strcmp(argv[i],"--geometry")){
          XParseGeometry(argv[i+1],&x,&y,&width,&height);
       }
	   else if (!strcmp(argv[i],"-fg")){
		  sprintf(color,"%s",argv[i+1]);
	   }
	   else if (!strcmp(argv[i],"-bg")){
		  sprintf(color_bg,"%s",argv[i+1]);
	   }		
	   else if (!strcmp(argv[i],"-f")){
          sprintf(myFont,"%s",argv[i+1]);
       }
	   else if (!strcmp(argv[i],"-s")){
		  sprintf(spool_file,"%s",argv[i+1]);
	   }
	   else if (!strcmp(argv[i],"-u")){
		  time_till_check = atoi(argv[i+1]);
	   } 
	   else if (!strcmp(argv[i],"-b")){
		  draw_border = 1;
	   }
 	   else if (!strcmp(argv[i],"+b")){
	   	  draw_border = 0;
	   }
  	   else if (!strcmp(argv[i],"-c")){
		  center = 1;
   	   }
 	   else if (!strcmp(argv[i],"+c")){
	   	  center = 0;
	   }
	   else if (!strcmp(argv[i],"-tr")){
	   	  transparent = 1;
	   }
	   else if (!strcmp(argv[i],"-x")){
	   	  fix = 1;
	   }
	   else if (!strcmp(argv[i],"-h") || !strcmp(argv[i],"--help")) {
          printf("usage: BadMail [--geometry -tr -x -fg -bg -f[ont] -s[pool file] -u[pdate time in seconds] [+-]c[enter] [+-]b[order]]\n");
          exit(0);
       }
    }
	
	last_check -= time_till_check;
	last_stat = time(NULL);

	dpy = XOpenDisplay(NULL);
  	scrn = DefaultScreen(dpy);
	root = RootWindow(dpy,scrn);	
	win_create();
	XStoreName(dpy,win,window_name);
	gc = XCreateGC(dpy,win,0,NULL);
	gc2 = XCreateGC(dpy,win,0,NULL);
	gc3 = XCreateGC(dpy,win,0,NULL);
	gc4 = XCreateGC(dpy,win,0,NULL);

	XSetForeground(dpy,gc,GetColor(color));
	XSetForeground(dpy,gc2,GetColor(color_bg));
	XSetBackground(dpy,gc,GetColor(color_bg));
	XSetBackground(dpy,gc2,GetColor(color));
	if(draw_border){
		XSetForeground(dpy,gc3,MakeBorderColor(color_bg,1));
		XSetForeground(dpy,gc4,MakeBorderColor(color_bg,0));
	}

	font = XLoadFont(dpy, myFont);
	font_ = XLoadQueryFont(dpy, myFont);	
  	
	XSetFont(dpy, gc, font);
  	XSetFont(dpy, gc2, font);

	// If transparent
	if(transparent)
		GetBackground();

	XMapWindow(dpy,win);

	while(1){ 
		// Stat the file, if changed - update now
		struct stat buf;
		
		if(stat(spool_file, &buf)){
			static int rep;
			if(!rep){ 
				fprintf(stderr, "BadMail: Can't stat spool.\n");
				exit(0);
			}
		}
			
		if(buf.st_mtime != last_stat){ 
			last_check = time(NULL) - time_till_check;
			last_stat = buf.st_mtime;	
		}

   		drawBadMail();
		XFlush(dpy);   
  		sleep(1);   
	}
	return 0;
}
